/*---------- REXX: HeartBeat (HEARTBT) -----------------------------*/ 
/*                                                                  */ 
/*  The process Reads a control file containing several dates:      */ 
/*    (1) HC_Time_Before    - Date/Time Stamp before BTEQ Begins    */ 
/*    (2) HC_Time_After     - Date/Time Stamp after BTEQ Ends       */ 
/*    (3) HC_Time_Missed    - Date/Time Stamp when miss reported    */ 
/*    (4) HC_Maint_Window   - Flag indicates Teradata is Down       */ 
/*    (5) HC_Missed_Beats   - Flag indicates heartbeat was missed   */ 
/*    (6) HC_Matched_Beats  - Counter that that resets notification */ 
/*                                                                  */ 
/*  The process EXPECTS two(2) parameters be supplied at Run Time:  */ 
/*    'A' or 'AFTER'        - Modify HC_Time_After                  */ 
/*    'B' or 'BEFORE'       - Modify HC_Time_Before                 */ 
/*    'C' or 'CHECK'        - Check to see if Heartbeat was Missed  */ 
/*    'S' or 'STARTMAINT'   - Start of Maintenance/Outage Window    */ 
/*    'F' or 'FINISHMAINT'  - Finish Maintenance/Outage Window      */ 
/*                                                                  */ 
/*  The second parameter is a fully qualifed file name              */ 
/*    for the HeartBeat TimeStamp Data file                         */ 
/*                                                                  */ 
/*  The process RETURNS the following Values:                       */ 
/*      0  -  Successful Completion                                 */ 
/*      4  -  Invalid Parameter (Not 'A', 'B', 'C' or 'M')          */ 
/*     12  -  Heart Beat Dataset not Allocated                      */ 
/*     16  -  Heart Beat has been missed                            */ 
/*      8  -  Maintenance Window active                             */ 
/*                                                                  */ 
/*------------------------------------------------------------------*/ 

ADDRESS TSO 

ARG Args 
PARSE UPPER VAR Args Flag dsName . 
if dsName = '' THEN 
  EXIT 12 
dsName = "'"dsName"'" 

HC_Maint_Window = 'FALSE'        /* Set default Value */ 

Flag = SUBSTR(Flag, 1, 1) 
SELECT 
  WHEN Flag = 'A' THEN NOP 
  WHEN Flag = 'B' THEN NOP 
  WHEN Flag = 'C' THEN NOP 
  WHEN Flag = 'S' THEN NOP 
  WHEN Flag = 'F' THEN NOP 
  OTHERWISE 
    EXIT 4 
  END 

IF LISTDSI(dsName) <> 0 THEN 
  EXIT 12 
"ALLOC DDN(heartbt) OLD DSN("dsName") REUSE" 

"NEWSTACK" 
SIGNAL ON ERROR 
"EXECIO * DISKR heartbt (STEM line. FINIS)" 
SIGNAL OFF ERROR 

DO I = 1 TO line.0 
  if INDEX(line.I, '=') = 0 THEN DO 
    QUEUE line.I 
    ITERATE I 
    END 
  PARSE VAR line.I Str '=' . 
  SELECT 
    WHEN Str = 'HC_Time_After'     THEN NOP 
    WHEN Str = 'HC_Time_Before'    THEN NOP 
    WHEN Str = 'HC_Time_Missed'    THEN NOP 
    WHEN Str = 'HC_Missed_Beats'   THEN NOP 
    WHEN Str = 'HC_Matched_Beats'  THEN NOP 
    WHEN Str = 'HC_Maint_Window'   THEN NOP 
    WHEN Str = 'Return_Code'       THEN NOP 
    OTHERWISE DO 
      QUEUE line.I 
      ITERATE I 
      END 
    END 
  INTERPRET line.I 
  END 

CurrDate = DATE('S') 
CurrTime = TIME() 
PARSE VAR CurrTime hh ':' mm ':' ss 
CurrTime = RIGHT('0'hh, 2)""RIGHT('0'mm, 2)""RIGHT('0'ss, 2) 
TimeStamp = CurrDate"@"CurrTime 
Return_Code = 0 

SELECT 
  WHEN Flag = 'F' THEN DO 
    HC_Maint_Window  = 'FALSE' 
    HC_Matched_Beats = 0 
    HC_Missed_Beats  = 0 
    END 

  WHEN Flag = 'S' THEN  HC_Maint_Window = 'TRUE' 
  WHEN Flag = 'A' THEN  HC_Time_After  = TimeStamp 
  WHEN Flag = 'B' THEN  HC_Time_Before = TimeStamp 
  WHEN Flag = 'C' THEN DO 
       HC_Matched_Beats = HC_Matched_Beats * 1 
       HC_Missed_Beats  = HC_Missed_Beats * 1 
       IF HC_Time_Before <= HC_Time_After THEN DO 
         IF HC_Matched_Beats > 0 THEN 
           HC_Matched_Beats = HC_Matched_Beats - 1 
         ELSE DO 
           IF HC_Missed_Beats > 0 THEN 
             HC_Missed_Beats = 0 
           END 
         END 
       ELSE DO 
         IF HC_Missed_Beats = 0 THEN DO 
           Return_Code = 16 
           HC_Time_Missed = TimeStamp 
           END 
         HC_Missed_Beats = 1 
         HC_Matched_Beats = 3 
         END 
       END 
  OTHERWISE             NOP 
  END 

/* Bypass during Maintenance & Outage Windows */ 
IF HC_Maint_Window = 'TRUE' 
  THEN Return_Code = 8 

QUEUE "HC_Missed_Beats     = "HC_Missed_Beats 
QUEUE "HC_Matched_Beats    = "HC_Matched_Beats 
QUEUE "HC_Time_Before      = "HC_Time_Before 
QUEUE "HC_Time_After       = "HC_Time_After 
QUEUE "HC_Time_Missed      = "HC_Time_Missed 
QUEUE "HC_Maint_Window     = "HC_Maint_Window 
QUEUE "Return_Code         = "Return_Code 

"EXECIO "QUEUED()" DISKW heartbt (FINIS)" 
"FREE DDN(heartbt)" 
"DELSTACK" 

IF Return_Code > 0 THEN DO 
  SAY "Args        = '"Args"'" 
  SAY "Return_Code = "Return_Code 
  END 

EXIT Return_Code 

/*---------- ERROR ROUTINE ------------------------------------------*/ 
ERROR: 
  EXIT 12 
